@@ -1,21 +1,20 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 from decimal import Decimal
+
 from dateutil.relativedelta import relativedelta
 
-from sql import Column
-
+from trytond import backend
 from trytond.i18n import gettext
 from trytond.model import (
-    ModelView, ModelSQL, DeactivableMixin, fields, sequence_ordered)
-from trytond import backend
+    DeactivableMixin, ModelSQL, ModelView, fields, sequence_ordered)
+from trytond.modules.currency.fields import Monetary
+from trytond.pool import Pool
 from trytond.pyson import Eval
 from trytond.transaction import Transaction
-from trytond.pool import Pool
-from trytond.wizard import Wizard, StateView, Button
-from trytond.config import config
+from trytond.wizard import Button, StateView, Wizard
 
-from .exceptions import PaymentTermValidationError, PaymentTermComputeError
+from .exceptions import PaymentTermComputeError, PaymentTermValidationError
 
 
 class PaymentTerm(DeactivableMixin, ModelSQL, ModelView):
@@ -32,34 +31,31 @@ class PaymentTerm(DeactivableMixin, ModelSQL, ModelView):
         cls._order.insert(0, ('name', 'ASC'))
 
     @classmethod
-    def validate(cls, terms):
-        super(PaymentTerm, cls).validate(terms)
+    def validate_fields(cls, terms, field_names):
+        super().validate_fields(terms, field_names)
+        cls.check_remainder(terms, field_names)
+
+    @classmethod
+    def check_remainder(cls, terms, field_names=None):
+        if field_names and 'lines' not in field_names:
+            return
         for term in terms:
-            term.check_remainder()
+            if not term.lines or not term.lines[-1].type == 'remainder':
+                raise PaymentTermValidationError(gettext(
+                        'account_invoice'
+                        '.msg_payment_term_missing_last_remainder',
+                        payment_term=term.rec_name))
 
-    def check_remainder(self):
-        if not self.lines or not self.lines[-1].type == 'remainder':
-            raise PaymentTermValidationError(
-                gettext('account_invoice'
-                    '.msg_payment_term_missing_last_remainder',
-                    payment_term=self.rec_name))
-
-    def compute(self, amount, currency, date=None):
+    def compute(self, amount, currency, date):
         """Calculate payment terms and return a list of tuples
         with (date, amount) for each payment term line.
 
         amount must be a Decimal used for the calculation.
-        If specified, date will be used as the start date, otherwise current
-        date will be used.
         """
         # TODO implement business_days
         # http://pypi.python.org/pypi/BusinessHours/
-        Date = Pool().get('ir.date')
-
         sign = 1 if amount >= Decimal(0) else -1
         res = []
-        if date is None:
-            date = Date.today()
         remainder = amount
         for line in self.lines:
             value = line.get_value(remainder, amount, currency)
@@ -99,24 +95,23 @@ class PaymentTermLine(sequence_ordered(), ModelSQL, ModelView):
         states={
             'invisible': ~Eval('type').in_(['percent', 'percent_on_total']),
             'required': Eval('type').in_(['percent', 'percent_on_total']),
-            }, depends=['type'])
+            })
     divisor = fields.Numeric('Divisor', digits=(10, 14),
         states={
             'invisible': ~Eval('type').in_(['percent', 'percent_on_total']),
             'required': Eval('type').in_(['percent', 'percent_on_total']),
-            }, depends=['type'])
-    amount = fields.Numeric('Amount', digits=(16, Eval('currency_digits', 2)),
+            })
+    amount = Monetary(
+        "Amount", currency='currency', digits='currency',
         states={
             'invisible': Eval('type') != 'fixed',
             'required': Eval('type') == 'fixed',
-            }, depends=['type', 'currency_digits'])
+            })
     currency = fields.Many2One('currency.currency', 'Currency',
         states={
             'invisible': Eval('type') != 'fixed',
             'required': Eval('type') == 'fixed',
-            }, depends=['type'])
-    currency_digits = fields.Function(fields.Integer('Currency Digits'),
-        'on_change_with_currency_digits')
+            })
     relativedeltas = fields.One2Many(
         'account.invoice.payment_term.line.delta', 'line', 'Deltas')
 
@@ -125,24 +120,6 @@ class PaymentTermLine(sequence_ordered(), ModelSQL, ModelView):
         super().__setup__()
         cls.__access__.add('payment')
 
-    @classmethod
-    def __register__(cls, module_name):
-        sql_table = cls.__table__()
-        super(PaymentTermLine, cls).__register__(module_name)
-        cursor = Transaction().connection.cursor()
-        table = cls.__table_handler__(module_name)
-
-        # Migration from 3.8: rename percentage into ratio
-        if table.column_exist('percentage'):
-            cursor.execute(*sql_table.update(
-                    columns=[sql_table.ratio],
-                    values=[sql_table.percentage / 100]))
-            table.drop_column('percentage')
-
-    @staticmethod
-    def default_currency_digits():
-        return 2
-
     @staticmethod
     def default_type():
         return 'remainder'
@@ -178,12 +155,6 @@ class PaymentTermLine(sequence_ordered(), ModelSQL, ModelView):
             self.ratio = self.round(1 / self.divisor,
                 self.__class__.ratio.digits[1])
 
-    @fields.depends('currency')
-    def on_change_with_currency_digits(self, name=None):
-        if self.currency:
-            return self.currency.digits
-        return 2
-
     def get_date(self, date):
         for relativedelta_ in self.relativedeltas:
             date += relativedelta_.get()
@@ -208,15 +179,16 @@ class PaymentTermLine(sequence_ordered(), ModelSQL, ModelView):
         return Decimal(number).quantize(quantize)
 
     @classmethod
-    def validate(cls, lines):
-        super(PaymentTermLine, cls).validate(lines)
-        cls.check_ratio_and_divisor(lines)
+    def validate_fields(cls, lines, field_names):
+        super().validate_fields(lines, field_names)
+        cls.check_ratio_and_divisor(lines, field_names)
 
     @classmethod
-    def check_ratio_and_divisor(cls, lines):
+    def check_ratio_and_divisor(cls, lines, field_names=None):
         "Check consistency between ratio and divisor"
-        # Use a copy because on_change will change the records
-        for line in cls.browse(lines):
+        if field_names and not (field_names & {'type', 'ratio', 'divisor'}):
+            return
+        for line in lines:
             if line.type not in ('percent', 'percent_on_total'):
                 continue
             if line.ratio is None or line.divisor is None:
@@ -260,22 +232,13 @@ class PaymentTermLineRelativeDelta(sequence_ordered(), ModelSQL, ModelView):
         transaction = Transaction()
         cursor = transaction.connection.cursor()
         pool = Pool()
-        Line = pool.get('account.invoice.payment_term.line')
         Month = pool.get('ir.calendar.month')
         Day = pool.get('ir.calendar.day')
         sql_table = cls.__table__()
-        line = Line.__table__()
         month = Month.__table__()
         day = Day.__table__()
         table_h = cls.__table_handler__(module_name)
 
-        # Migration from 4.0: rename long table
-        old_model_name = 'account.invoice.payment_term.line.relativedelta'
-        old_table = config.get(
-            'table', old_model_name, default=old_model_name.replace('.', '_'))
-        if backend.TableHandler.table_exist(old_table):
-            backend.TableHandler.table_rename(old_table, cls._table)
-
         # Migration from 5.0: use ir.calendar
         migrate_calendar = False
         if (backend.TableHandler.table_exist(cls._table)
@@ -291,19 +254,6 @@ class PaymentTermLineRelativeDelta(sequence_ordered(), ModelSQL, ModelView):
         super(PaymentTermLineRelativeDelta, cls).__register__(module_name)
 
         table_h = cls.__table_handler__(module_name)
-        line_table = Line.__table_handler__(module_name)
-
-        # Migration from 3.4
-        fields = ['day', 'month', 'weekday', 'months', 'weeks', 'days']
-        if any(line_table.column_exist(f) for f in fields):
-            columns = ([line.id.as_('line')]
-                + [Column(line, f) for f in fields])
-            cursor.execute(*sql_table.insert(
-                    columns=[sql_table.line]
-                    + [Column(sql_table, f) for f in fields],
-                    values=line.select(*columns)))
-            for field in fields:
-                line_table.drop_column(field)
 
         # Migration from 5.0: use ir.calendar
         if migrate_calendar:
@@ -366,14 +316,17 @@ class TestPaymentTermView(ModelView):
     __name__ = 'account.invoice.payment_term.test'
     payment_term = fields.Many2One('account.invoice.payment_term',
         'Payment Term', required=True)
-    date = fields.Date('Date')
-    amount = fields.Numeric('Amount', required=True,
-        digits=(16, Eval('currency_digits', 2)), depends=['currency_digits'])
+    date = fields.Date("Date", required=True)
+    amount = Monetary(
+        "Amount", currency='currency', digits='currency', required=True)
     currency = fields.Many2One('currency.currency', 'Currency', required=True)
-    currency_digits = fields.Integer('Currency Digits')
     result = fields.One2Many('account.invoice.payment_term.test.result',
         None, 'Result', readonly=True)
 
+    @classmethod
+    def default_date(cls):
+        return Pool().get('ir.date').today()
+
     @staticmethod
     def default_currency():
         pool = Pool()
@@ -382,34 +335,25 @@ class TestPaymentTermView(ModelView):
         if company:
             return Company(company).currency.id
 
-    @fields.depends('currency')
-    def on_change_with_currency_digits(self):
-        if self.currency:
-            return self.currency.digits
-        return 2
-
     @fields.depends('payment_term', 'date', 'amount', 'currency', 'result')
     def on_change_with_result(self):
         pool = Pool()
         Result = pool.get('account.invoice.payment_term.test.result')
         result = []
-        if (self.payment_term and self.amount and self.currency):
+        if (self.payment_term and self.amount and self.currency and self.date):
             for date, amount in self.payment_term.compute(
                     self.amount, self.currency, self.date):
                 result.append(Result(
                         date=date,
                         amount=amount,
-                        currency=self.currency,
-                        currency_digits=self.currency.digits))
-        self.result = result
-        return self._changed_values.get('result', [])
+                        currency=self.currency))
+        return result
 
 
 class TestPaymentTermViewResult(ModelView):
     'Test Payment Term'
     __name__ = 'account.invoice.payment_term.test.result'
     date = fields.Date('Date', readonly=True)
-    amount = fields.Numeric('Amount', readonly=True,
-        digits=(16, Eval('currency_digits', 2)), depends=['currency_digits'])
+    amount = Monetary(
+        "Amount", currency='currency', digits='currency', readonly=True)
     currency = fields.Many2One('currency.currency', "Currency")
-    currency_digits = fields.Integer('Currency Digits')
