@@ -1,6 +1,6 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 
-from .test_account_invoice import suite
+from .test_module import set_invoice_sequences
 
-__all__ = ['suite']
+__all__ = ['set_invoice_sequences']
